@@ -1,8 +1,8 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.3 2009/07/22 08:46:55 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.6 2009/09/25 07:35:16 t-ishii Exp $
  *
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
  * Copyright (c) 2003-2009	PgPool Global Development Group
@@ -121,7 +121,7 @@ ProcessInfo *pids;	/* shmem child pid table */
  * this is a two dimension array. i.e.:
  * con_info[pool_config->num_init_children][pool_config->max_pool]
  */
-ConnectionInfo *con_info;		
+ConnectionInfo *con_info;
 
 static int unix_fd;	/* unix domain socket fd */
 static int inet_fd;	/* inet domain socket fd */
@@ -247,7 +247,7 @@ int main(int argc, char **argv)
 					exit(1);
 				}
 				break;
-				
+
 			case 'n':	/* no detaching control ttys */
 				not_detach = 1;
 				break;
@@ -428,6 +428,14 @@ int main(int argc, char **argv)
 	}
 	*InRecovery = 0;
 
+	/*
+	 * We need to block signal here. Otherwise child might send some
+	 * signals, for example SIGUSR1(fail over).  Children will inherit
+	 * signal blocking but they do unblock signals at the very beginning
+	 * of process.  So this is harmless.
+	 */
+	POOL_SETMASK(&BlockSig);
+
 	/* fork the children */
 	for (i=0;i<pool_config->num_init_children;i++)
 	{
@@ -436,7 +444,7 @@ int main(int argc, char **argv)
 	}
 
 	/* set up signal handlers */
-	POOL_SETMASK(&BlockSig);
+
 	pool_signal(SIGTERM, exit_handler);
 	pool_signal(SIGINT, exit_handler);
 	pool_signal(SIGQUIT, exit_handler);
@@ -755,7 +763,7 @@ static void write_pid_file(void)
 		exit(1);
 	}
 	snprintf(pidbuf, sizeof(pidbuf), "%d", (int)getpid());
-	fwrite(pidbuf, strlen(pidbuf), 1, fd);
+	fwrite(pidbuf, strlen(pidbuf)+1, 1, fd);
 	if (fclose(fd))
 	{
 		pool_error("could not write pid file as %s. reason: %s",
@@ -771,7 +779,7 @@ static void write_pid_file(void)
 pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 {
 	pid_t pid;
-	
+
 	pid = fork();
 
 	if (pid == 0)
@@ -954,7 +962,7 @@ static void myunlink(const char* path)
 static void myexit(int code)
 {
 	int i;
-	
+
 	if (getpid() != mypid)
 		return;
 
@@ -975,7 +983,7 @@ static void myexit(int code)
 			pool_error("wait() failed. reason:%s", strerror(errno));
 		POOL_SETMASK(&UnBlockSig);
 	}
-	
+
 	myunlink(un_addr.sun_path);
 	myunlink(pcp_un_addr.sun_path);
 	myunlink(pool_config->pid_file_name);
@@ -1187,7 +1195,7 @@ static void failover(void)
 		return;
 	}
 
-	/* 
+	/*
 	 * if not in replication mode/master slave mode, we treat this a restart request.
 	 * otherwise we need to check if we have already failovered.
 	 */
@@ -1510,7 +1518,7 @@ system_db_health_check(void)
 	/* if SystemDB is already down, ignore */
 	if (SYSDB_STATUS == CON_UNUSED || SYSDB_STATUS == CON_DOWN)
 		return 0;
-	
+
 	if (*SYSDB_INFO->hostname == '\0')
 		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir);
 	else
@@ -1605,7 +1613,7 @@ static void reaper(void)
 			break;
 		} else {
 			pool_debug("child %d exits with status %d by signal %d", pid, status, WTERMSIG(status));
-		
+
 			/* look for exiting child's pid */
 			for (i=0;i<pool_config->num_init_children;i++)
 			{
@@ -1687,12 +1695,12 @@ pool_get_system_db_info(void)
 {
 	if (system_db_info == NULL)
 		return NULL;
-	
+
 	return system_db_info->info;
 }
 
 
-/* 
+/*
  * handle SIGUSR2
  * Wakeup all processes
  */
@@ -1704,10 +1712,10 @@ static void wakeup_children(void)
 
 static RETSIGTYPE wakeup_handler(int sig)
 {
-	POOL_SETMASK(&BlockSig);	
+	POOL_SETMASK(&BlockSig);
 	wakeup_request = 1;
 	write(pipe_fds[1], "\0", 1);
-	POOL_SETMASK(&UnBlockSig);	
+	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1807,7 +1815,7 @@ static void pool_sleep(unsigned int second)
 
 		r = pool_pause(&timeout);
 		POOL_SETMASK(&BlockSig);
-		if (r > 0)									
+		if (r > 0)
 			CHECK_REQUEST;
 		POOL_SETMASK(&UnBlockSig);
 		gettimeofday(&current_time, NULL);
@@ -1862,7 +1870,7 @@ static int trigger_failover_command(int node, const char *command_line)
 		return -1;
 	}
 	exec_cmd = init_string("");
-	
+
 	while (*command_line)
 	{
 		if (*command_line == '%')
