@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/prj/pg/ipa-pgpool2/cvsroot/ipa-pgpool2/pgpool_src/pool_auth.c,v 1.12 2006/07/28 04:50:26 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_auth.c,v 1.15.2.2 2009/09/23 02:03:12 t-ishii Exp $
  *
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -36,6 +36,8 @@
 #include <errno.h>
 #include <string.h>
 
+#define AUTHFAIL_ERRORCODE "28000"
+
 static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key, int protoMajor);
 static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth, int protoMajor);
 static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reauth, int protoMajor);
@@ -51,6 +53,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	int key;
 	int protoMajor;
 	int length;
+	int authkind;
 	int i;
 	StartupPacket *sp;
 
@@ -81,9 +84,14 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	}
 
 	/*
-	 * message length (v3 only) */
+	 * message length (v3 only)
+	 */
 	if (protoMajor == PROTO_MAJOR_V3 && pool_read_message_length(cp) < 0)
 	{
+		pool_error("Failed to read the authentication packet length. \
+This is likely caused by the inconsistency of auth method among DB nodes. \
+In this case you can check the previous error messages (hint: length field) \
+from pool_read_message_length and recheck the pg_hba.conf settings.");
 		return -1;
 	}
 
@@ -98,22 +106,22 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	 * 5: md5 password
 	 * 6: scm credential
 	 *
-	 * in replication mode, we only supports  kind = 0, 3. this is because to "salt"
+	 * in replication mode, we only support  kind = 0, 3. this is because to "salt"
 	 * cannot be replicated.
 	 * in non replication mode, we support kind = 0, 3, 4, 5
 	 */
 
-	pid = pool_read_int(cp);
-	if (pid < 0)
+	authkind = pool_read_int(cp);
+	if (authkind < 0)
 	{
-		pool_error("pool_do_auth: read pid failed");
+		pool_error("pool_do_auth: read auth kind failed");
 		return -1;
 	}
 
-	pid = ntohl(pid);
+	authkind = ntohl(authkind);
 
 	/* trust? */
-	if (pid == 0)
+	if (authkind == 0)
 	{
 		int msglen;
 
@@ -134,15 +142,18 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	}
 
 	/* clear text password authentication? */
-	else if (pid == 3)
+	else if (authkind == 3)
 	{
 		for (i=0;i<NUM_BACKENDS;i++)
 		{
+			if (!VALID_BACKEND(i))
+				continue;
+
 			pool_debug("trying clear text password authentication");
 
-			pid = do_clear_text_password(CONNECTION(cp, i), frontend, 0, protoMajor);
+			authkind = do_clear_text_password(CONNECTION(cp, i), frontend, 0, protoMajor);
 
-			if (pid < 0)
+			if (authkind < 0)
 			{
 				pool_error("do_clear_text_password failed in slot %d", i);
 				return -1;
@@ -151,15 +162,18 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	}
 
 	/* crypt authentication? */
-	else if (pid == 4)
+	else if (authkind == 4)
 	{
 		for (i=0;i<NUM_BACKENDS;i++)
 		{
+			if (!VALID_BACKEND(i))
+				continue;
+
 			pool_debug("trying crypt authentication");
 
-			pid = do_crypt(CONNECTION(cp, i), frontend, 0, protoMajor);
+			authkind = do_crypt(CONNECTION(cp, i), frontend, 0, protoMajor);
 
-			if (pid < 0)
+			if (authkind < 0)
 			{
 				pool_error("do_crypt_text_password failed in slot %d", i);
 				return -1;
@@ -168,15 +182,28 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	}
 
 	/* md5 authentication? */
-	else if (pid == 5)
+	else if (authkind == 5)
 	{
+		if (!RAW_MODE && NUM_BACKENDS > 1)
+		{
+			pool_send_error_message(frontend, protoMajor, AUTHFAIL_ERRORCODE,
+									"MD5 authentication is unsupported in replication, master-slave and parallel modes.",
+									"",
+									"check pg_hba.conf",
+									__FILE__, __LINE__);
+			return -1;
+		}
+
 		for (i=0;i<NUM_BACKENDS;i++)
 		{
+			if (!VALID_BACKEND(i))
+				continue;
+
 			pool_debug("trying md5 authentication");
 
-			pid = do_md5(CONNECTION(cp, i), frontend, 0, protoMajor);
+			authkind = do_md5(CONNECTION(cp, i), frontend, 0, protoMajor);
 
-			if (pid < 0)
+			if (authkind < 0)
 			{
 				pool_error("do_md5failed in slot %d", i);
 				return -1;
@@ -184,11 +211,17 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		}
 	}
 
-	if (pid != 0)
+	else
 	{
-		pool_error("pool_do_auth: backend does not return authenticaton ok");
+		pool_error("pool_do_auth: unsupported auth kind received: %d", authkind);
 		return -1;
-	} 
+	}
+
+	if (authkind != 0)
+	{
+		pool_error("pool_do_auth: backend does not return authentication ok");
+		return -1;
+	}
 
 	/*
 	 * authentication ok. now read pid and secret key from the
@@ -199,7 +232,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		kind = pool_read_kind(cp);
 		if (kind < 0)
 		{
-			pool_error("pool_do_auth: failed to read kind before backendkeydata");
+			pool_error("pool_do_auth: failed to read kind before BackendKeyData");
 			return -1;
 		}
 		else if (kind == 'K')
@@ -294,7 +327,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 				pool_error("pool_do_auth: failed to read pid in slot %d", i);
 				return -1;
 			}
-			CONNECTION_SLOT(cp, i)->pid = pid;
+
+			CONNECTION_SLOT(cp, i)->pid = cp->info[i].pid = pid;
 
 			/* read key */
 			if (pool_read(CONNECTION(cp, i), &key, sizeof(key)) < 0)
@@ -302,7 +336,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 				pool_error("pool_do_auth: failed to read key in slot %d", i);
 				return -1;
 			}
-			CONNECTION_SLOT(cp, i)->key = key;
+			CONNECTION_SLOT(cp, i)->key = cp->info[i].key = key;
+
 		}
 	}
 
@@ -337,7 +372,7 @@ int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			/* clear text password */
 			status = do_clear_text_password(MASTER(cp), frontend, 1, protoMajor);
 			break;
-			
+
 		case 4:
 			/* crypt password */
 			status = do_crypt(MASTER(cp), frontend, 1, protoMajor);
@@ -349,7 +384,7 @@ int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 			break;
 
 		default:
-			pool_error("pool_do_reauth: unknown authentication request code %d", 
+			pool_error("pool_do_reauth: unknown authentication request code %d",
 					   MASTER(cp)->auth_kind);
 			return -1;
 	}
@@ -663,13 +698,13 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int rea
 		pool_debug("size: %d saved_size: %d", (ntohl(size) - 4), backend->pwd_size);
 		if ((ntohl(size) - 4) != backend->pwd_size)
 		{
-			pool_debug("do_crypt: password size does not match in re-authetication");
+			pool_debug("do_crypt: password size does not match in re-authentication");
 			return -1;
 		}
 
 		if (memcmp(password, backend->password, backend->pwd_size) != 0)
 		{
-			pool_debug("do_crypt: password does not match in re-authetication");
+			pool_debug("do_crypt: password does not match in re-authentication");
 			return -1;
 		}
 
@@ -831,13 +866,13 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reaut
 	{
 		if ((ntohl(size) - 4) != backend->pwd_size)
 		{
-			pool_debug("do_md5; password size does not match in re-authetication");
+			pool_debug("do_md5; password size does not match in re-authentication");
 			return -1;
 		}
 
 		if (memcmp(password, backend->password, backend->pwd_size) != 0)
 		{
-			pool_debug("do_md5; password does not match in re-authetication");
+			pool_debug("do_md5; password does not match in re-authentication");
 			return -1;
 		}
 
@@ -919,11 +954,19 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 	int length, length0;
 	int i;
 
-	length0 = 0;
+	/* read message from master node */
+	status = pool_read(CONNECTION(cp, MASTER_NODE_ID), &length0, sizeof(length0));
+	if (status < 0)
+	{
+		pool_error("pool_read_message_length: error while reading message length in slot %d", MASTER_NODE_ID);
+		return -1;
+	}
+	length0 = ntohl(length0);
+	pool_debug("pool_read_message_length: slot: %d length: %d", MASTER_NODE_ID, length0);
 
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (!VALID_BACKEND(i))
+		if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 		{
 			continue;
 		}
@@ -938,33 +981,26 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 		length = ntohl(length);
 		pool_debug("pool_read_message_length: slot: %d length: %d", i, length);
 
-		if (IS_MASTER_NODE_ID(i))
+		if (length != length0)
 		{
-			length0 = length;
-		}
-		else
-		{
-			if (length != length0)
-			{
-				pool_error("pool_read_message_length: message length (%d) in slot %d does not match with slot 0", length, i);
-				return -1;
-			}
+			pool_error("pool_read_message_length: message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0);
+			return -1;
 		}
 	}
 
-	if (length < 0)
+	if (length0 < 0)
 	{
 		pool_error("pool_read_message_length: invalid message length (%d)", length);
 		return -1;
 	}
 
-	return length;
+	return length0;
 }
 
 /*
  * read message length2 (V3 only)
  * unlike pool_read_message_length, this returns an array of message length.
- * the array is in the static storage, thus it will be destroyed by subsequent calls.
+ * The array is in the static storage, thus it will be destroyed by subsequent calls.
  */
 int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 {
@@ -973,11 +1009,20 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 	int i;
 	static int length_array[MAX_CONNECTION_SLOTS];
 
-	length0 = 0;
+	/* read message from master node */
+	status = pool_read(CONNECTION(cp, MASTER_NODE_ID), &length0, sizeof(length0));
+	if (status < 0)
+	{
+		pool_error("pool_read_message_length2: error while reading message length in slot %d", MASTER_NODE_ID);
+		return NULL;
+	}
 
+	length0 = ntohl(length0);
+	length_array[MASTER_NODE_ID] = length0;
+	pool_debug("pool_read_message_length2: master slot: %d length: %d", MASTER_NODE_ID, length0);
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (VALID_BACKEND(i))
+		if (VALID_BACKEND(i) && !IS_MASTER_NODE_ID(i))
 		{
 			status = pool_read(CONNECTION(cp, i), &length, sizeof(length));
 			if (status < 0)
@@ -989,16 +1034,9 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 			length = ntohl(length);
 			pool_debug("pool_read_message_length2: master slot: %d length: %d", i, length);
 
-			if (IS_MASTER_NODE_ID(i))
+			if (length != length0)
 			{
-				length0 = length;
-			}
-			else
-			{
-				if (length != length0)
-				{
-					pool_error("pool_read_message_length2: message length (%d) in slot %d does not match with slot 0", length, i);
-				}
+				pool_log("pool_read_message_length2: message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0);
 			}
 
 			if (length < 0)
@@ -1044,7 +1082,7 @@ signed char pool_read_kind(POOL_CONNECTION_POOL *cp)
 		{
 			if (kind != kind0)
 			{
-				pool_error("pool_read_kind: kind does not match between master(%d) slot[%d] (%d)",
+				pool_error("pool_read_kind: kind does not match between master(%x) slot[%d] (%x)",
 						   kind0, i, kind);
 				return -1;
 			}
@@ -1084,7 +1122,7 @@ int pool_read_int(POOL_CONNECTION_POOL *cp)
 		{
 			if (data != data0)
 			{
-				pool_error("pool_read_int: data does not match between between master(%d) slot[%d] (%d)",
+				pool_error("pool_read_int: data does not match between between master(%x) slot[%d] (%x)",
 						   data0, i, data);
 				return -1;
 			}
